#!/bin/sh -u
#
# Installation script for dco/c++ 3.6

usage()
{
  echo ""
  echo "${0} usage:"
  echo "-silent                Means perform a silent installation"
  echo "                       In this case, the -accept and -installdir"
  echo "                       switches must also be used"
  echo "-accept                means that you accept the NAG licence agreement"
  echo "-installdir=DIR        Means that the library will be installed under"
  echo "                       directory DIR"
  exit
}

get_installdir()
{
  installdir=""
  while [ "${installdir}" = "" ]; do
    echo "> "
    read ans
    if [ $ans ]; then
      case $ans in
      *) installdir=$ans ;;
      esac
    else
      installdir=${defaultinstalldir}
    fi
  done
  # Replace any ~ by ${HOME} otherwise you end up with a
  # subdirectory named ~ (dangerous if you then try to remove it!)
  installdir=`echo ${installdir} | sed -e "s%~%${HOME}%g"`
}

showLicence()
{
  licence_agreement_file=${tooldir}/nag_tsandcs.txt
  if [ -f ${licence_agreement_file} ]; then
    more ${licence_agreement_file}
  else
    echo "Cannot find licence agreement file ${licence_agreement_file}."
    echo "dco/c++ not installed."
    exit
  fi

  reply=""
  while [ "${reply}" != "accept" -a "${reply}" != "decline" ]; do
    echo "[accept/decline]? :"
    read reply
    reply=`echo ${reply} | tr '[:upper:]' '[:lower:]'`
  done
}

# Echoes a message unless in silent install mode
echoifnotsilent()
{
  if [ "${silentinstall}" = "0" ]; then
    echo ${1}
  fi
}

umask 022

toolname=`basename "${0}"`

# Get the full path of the installation material directory
fromdir=`pwd`
tooldir=`dirname "${0}"`
cd "${tooldir}"
tooldir=`pwd`
cd ${fromdir}

# Check for any switch arguments
doneswitches=0
acceptlicence=0
silentinstall=0
installdir=""
while test "${doneswitches}" = "0"
do
  case "${1-}" in
    -accept*)
      acceptlicence=1
      shift;;
    -installdir=*)
      installdir=`echo ${1} | sed -e 's%.*=%%' -e "s%~%${HOME}%g"`
    shift;;
    -silent)
      silentinstall=1
      shift;;
    -*)
      echo "${toolname}: unrecognised switch: ${1}"
      usage
      shift;;
    *)
      doneswitches=1;;
  esac
done

echoifnotsilent "               dco/c++ Library Installation  "
echoifnotsilent "               ============================  "
echoifnotsilent ""
echoifnotsilent "This script installs dco/c++ 3.6 (DCL6I36NGL)."

if [ "${silentinstall}" = "1" ]; then
  if [ "${acceptlicence}" = "0" -o "${installdir}" = "" ]; then
    echo "${0}: ERROR: if you use the -silent switch, you must also"
    echo "use the -accept and -installdir switches"
    usage
  fi
fi

tarextractflags=poxf

# Default installation directory is under ${HOME} if that directory already
# exists. Otherwise, default installation directory is under /opt.
if [ -d "${HOME}" ]; then
  defaultinstalldir=${HOME}/NAG
else
  defaultinstalldir=/opt/NAG
fi

# Install the library if it is required
if [ "${installdir}" = "" ]; then
  echo
  echo "Do you wish to install dco/c++ 3.6? (yes/no):"
  read instlib
  if [ "$instlib" = "y" -o "$instlib" = "ye" -o "$instlib" = "yes" ] ; then
    instlib="y"
  else
    echo "Installation of dco/c++ not confirmed"
    instlib="n"
  fi
else
  instlib="y"
fi

if [ "$instlib" = "y" ] ; then
  if [ "${acceptlicence}" = "0" ]; then
    showLicence
    if [ "${reply}" != "accept" ]; then
      echo "Installation declined. dco/c++ not installed."
      exit
    fi
  fi

  libtarfile=${tooldir}/dcl6i36ngl_v362.tar.gz
  if [ ! -f "${libtarfile}" ]; then
    echo "${0}: ERROR: Installation file ${libtarfile} does not exist."
    echo "Installation failed."
    exit
  fi

  if [ "${installdir}" = "" ]; then
    echo ""
    echo "Where do you want to install dco/c++ 3.6?"
    echo "Accept the default location or enter an alternative path. The"
    echo "directory will be created if it does not already exist."
    echo ""
    echo "If this installation is to be accessed by multiple users"
    echo "you may prefer to install in a common area such as /opt/NAG"
    echo "or /usr/local/NAG rather than the default location."
    echo ""
    echo "Enter installation location (or press return for ${defaultinstalldir}):"
    get_installdir
  fi

  echoifnotsilent ""
  echoifnotsilent "Installing dco/c++ material in subdirectory"
  echoifnotsilent "dcl6i36ngl_v362 of ${installdir}"
  echoifnotsilent ""

  # Ask user to confirm overwriting of previous installation, unless
  # in silent mode in which case we go ahead anyway.
  if [ -d ${installdir}/dcl6i36ngl_v362 -a "${silentinstall}" = "0" ]; then
    echo "Subdirectory ${installdir}/dcl6i36ngl_v362 already exists."
    echo "Do you wish to overwrite it? (yes/no):"
    read ans
    ans=`echo ${ans} | tr '[:upper:]' '[:lower:]'`
    if [ "$ans" = "y" -o "$ans" = "ye" -o "${ans}" = "yes" ] ; then
      echo "Previous installation will be overwritten"
    else
      instlib="n"
      echo "Previous installation will not be overwritten"
    fi
  fi

  if [ "${instlib}" = "y" ]; then
    if [ ! -d ${installdir} ]; then
      mkdir -p ${installdir}
      if [ $? -ne 0 ]; then
        echo "${0}: ERROR: cannot create installation directory"
        echo "  ${installdir}"
        echo "- installation failed."
        exit
      fi
    fi

    touch ${installdir}/temp.temp 2>/dev/null
    if [ -f ${installdir}/temp.temp ]; then
      rm ${installdir}/temp.temp
    else
      echo "${0}: ERROR: you do not have write permission for directory:"
      echo "  ${installdir}"
      exit
    fi
    echoifnotsilent "cd ${installdir}"
    cd ${installdir}
    fullpathinstalldir=`pwd`
    echoifnotsilent "gunzip -c ${libtarfile} | tar ${tarextractflags} -"
    gunzip -c ${libtarfile} | tar ${tarextractflags} -
  fi
fi

if [ "$instlib" = "y" ] ; then
  if [ "`echo dcl6i36ngl_v362 | sed -e 's/.*\(.\)/\1/'`" = "l" ]; then
    echoifnotsilent ""
    echoifnotsilent "The use of this product is controlled by the Kusari license"
    echoifnotsilent "management system. In order to enable the software, NAG will"
    echoifnotsilent "issue you with a license key. See the Installer's Note"
    echoifnotsilent "  ${fullpathinstalldir}/dcl6i36ngl_v362/doc/in.html"
    echoifnotsilent "for more information on using license keys."
  fi

  # Tell people to read the user guide
  echoifnotsilent ""
  echoifnotsilent "For instructions on how to compile and link programs to this"
  echoifnotsilent "version of dco/c++, see the User Guide:"
  echoifnotsilent "  ${fullpathinstalldir}/dcl6i36ngl_v362/doc/userguide.pdf"
  echoifnotsilent ""
fi

# Go back to where we started
echoifnotsilent "cd ${fromdir}"
cd ${fromdir}

echoifnotsilent "====== dco/c++ installation complete ======"
